"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var requireModule_1 = require("../requireModule");
var CssProcessorPlugin = (function () {
    function CssProcessorPlugin() {
    }
    CssProcessorPlugin.prototype.configure = function (builder, spin) {
        var stack = builder.stack;
        var dev = spin.dev;
        if (stack.hasAll('webpack')) {
            var createRule = void 0;
            var rules = [];
            var postCssLoader_1 = requireModule_1.default.probe('postcss-loader');
            if (stack.hasAny('server')) {
                createRule = function (ext, ruleList) { return ({
                    test: new RegExp("\\." + ext + "$"),
                    use: dev
                        ? [
                            { loader: requireModule_1.default.resolve('isomorphic-style-loader') },
                            { loader: requireModule_1.default.resolve('css-loader'), options: { sourceMap: true } }
                        ]
                            .concat(postCssLoader_1 ? { loader: postCssLoader_1, options: { sourceMap: true } } : [])
                            .concat(ruleList)
                        : [{ loader: requireModule_1.default.resolve('ignore-loader') }]
                }); };
            }
            else if (stack.hasAny('web')) {
                var ExtractTextPlugin_1;
                if (!dev) {
                    ExtractTextPlugin_1 = requireModule_1.default('extract-text-webpack-plugin');
                }
                createRule = function (ext, ruleList) {
                    var plugin;
                    if (!dev) {
                        plugin = new ExtractTextPlugin_1({ filename: "[name].[contenthash]_" + ext + ".css" });
                        builder.config.plugins.push(plugin);
                    }
                    return {
                        test: new RegExp("\\." + ext + "$"),
                        use: dev
                            ? [
                                { loader: requireModule_1.default.resolve('style-loader') },
                                { loader: requireModule_1.default.resolve('css-loader'), options: { sourceMap: true, importLoaders: 1 } }
                            ]
                                .concat(postCssLoader_1 ? { loader: postCssLoader_1, options: { sourceMap: true } } : [])
                                .concat(ruleList)
                            : plugin.extract({
                                fallback: requireModule_1.default.resolve('style-loader'),
                                use: [requireModule_1.default.resolve('css-loader'), requireModule_1.default.resolve('postcss-loader')].concat(ruleList ? ruleList.map(function (rule) { return rule.loader; }) : [])
                            })
                    };
                };
            }
            if (createRule && stack.hasAny('css')) {
                rules.push(createRule('css', []));
            }
            if (createRule && stack.hasAny('sass')) {
                rules.push(createRule('scss', [{ loader: requireModule_1.default.resolve("sass-loader"), options: { sourceMap: true } }]));
            }
            if (createRule && stack.hasAny('less')) {
                rules.push(createRule('less', [{ loader: requireModule_1.default.resolve("less-loader"), options: { sourceMap: true } }]));
            }
            builder.config = spin.merge(builder.config, {
                module: {
                    rules: rules
                }
            });
        }
    };
    return CssProcessorPlugin;
}());
exports.default = CssProcessorPlugin;
//# sourceMappingURL=CssProcessorPlugin.js.map